'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeImporter = makeImporter;
exports.makeSyncImporter = makeSyncImporter;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Search for the likely absolute path from a relative path using known paths from compilation
 */
function findAbsolutePath(extractions, relativePath) {
  var filenames = Object.keys(extractions);

  for (var i = 0; i < filenames.length; i++) {
    if (filenames[i].match(relativePath)) {
      return filenames[i];
    }
  }

  return relativePath;
}

/**
 * Get the absolute file path for a relative @import like './sub/file.scsss'
 * If the @import is made from a raw data section a best guess path is returned
 */
function getImportAbsolutePath(extractions, url, prev) {
  var absolutePath = _path2.default.posix.join(_path2.default.posix.dirname(prev), url);
  var extension = _path2.default.posix.extname(prev);

  // Ensure that both @import 'file' and @import 'file.scss' is mapped correctly
  if (_path2.default.posix.extname(absolutePath) !== extension) {
    absolutePath += extension;
  }

  if (prev === 'stdin') {
    absolutePath = findAbsolutePath(extractions, absolutePath);
  }

  return absolutePath;
}

/**
 * Get the resulting source and path for a given @import request
 */
function getImportResult(extractions, url, prev) {
  var absolutePath = getImportAbsolutePath(extractions, url, prev);
  var contents = extractions[absolutePath].injectedData;

  return { file: absolutePath, contents: contents };
}

/**
 * Create an importer that will resolve @import directives with the injected
 * data found in provided extractions object
 */
function makeImporter(extractions) {
  return function (url, prev, done) {
    try {
      var result = getImportResult(extractions, url, prev);
      done(result);
    } catch (err) {
      done(err);
    }
  };
}

/**
 * Create a synchronous importer that will resolve @import directives with the injected
 * data found in provided extractions object
 */
function makeSyncImporter(extractions) {
  return function (url, prev) {
    try {
      var result = getImportResult(extractions, url, prev);
      return result;
    } catch (err) {
      // note: importer must return errors
      return err;
    }
  };
}